unit cdsqreg;

interface

uses
    DB, DBTables, DBConsts, BdeProv, DBClient;

procedure Register;

implementation

uses
    Dialogs, ClientDataSetQuery, SysUtils, Classes, BDEConst, UpdSQLEd,
    QBndDlg, DsgnIntf, DBReg;

var
  QueryEditor, PrevDataSetEditor: TComponentEditorClass;
  QBEVerb: String;

type
  TClientDataSetQueryEditor = class(TComponentEditor)
  private
    FPrevEditor: TComponentEditor;
    FQueryEditor: TComponentEditor;
    function GetQBEVerbIndex: Integer;
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
    constructor Create(AComponent: TComponent; ADesigner:
      {$IFDEF VER120}IFormDesigner{$ELSE}TFormDesigner{$ENDIF}  ); override;
    destructor destroy; override;
  end;

(* Portions of the following code are copied from RxLib *)

type
  TParamsProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

type
  TDatabaseNameProperty = class(TDBStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

type
  TSessionNameProperty = class(TDBStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

procedure TSessionNameProperty.GetValueList(List: TStrings);
begin
  Sessions.GetSessionNames(List);
end;

procedure TDatabaseNameProperty.GetValueList(List: TStrings);
begin
  with (GetComponent(0) as TClientDataSetQuery) do
       Query.DBSession.GetDatabaseNames(List);
end;

procedure TParamsProperty.Edit;
var
  NewParams: TParams;
  Query: TQuery;
begin
  inherited;
  NewParams := TParams.Create;
  try
     Query:=(GetComponent(0) as TClientDataSetQuery).Query;
     NewParams.Assign(Query.Params);
     if EditQueryParams(Query, NewParams) and not NewParams.IsEqual(Query.Params) then
     begin
       Modified;
       Query.Close;
       Query.Params:=NewParams;
     end;
  finally
    NewParams.Free;
  end;
end;

function TParamsProperty.GetValue: string;
begin
  Result := Format('(%s)', [TParams.ClassName]);
end;

function TParamsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

constructor TClientDataSetQueryEditor.Create(AComponent: TComponent;
  ADesigner:{$IFDEF VER120}IFormDesigner{$ELSE}TFormDesigner{$ENDIF});
begin
  inherited;
  FPrevEditor := PrevDataSetEditor.Create(AComponent, ADesigner);
  FQueryEditor:= QueryEditor.Create((AComponent as TClientDataSetQuery).Query, ADesigner);
end;

destructor TClientDataSetQueryEditor.Destroy;
begin
  inherited;
  FQueryEditor.Free;
  FPrevEditor.free;
end;

function TClientDataSetQueryEditor.GetQBEVerbIndex: Integer;
var
   VerbCount, i: Integer;
begin
  Result:=-1;
  VerbCount:=FQueryEditor.GetVerbCount;
  for i:=0 to VerbCount-1 do
      if FQueryEditor.GetVerb(i)=QBEVerb then
      begin
           Result:=i;
           Break;
      end;
end;

procedure TClientDataSetQueryEditor.ExecuteVerb(Index: Integer);
var
   QBEVerbIndex: Integer;
begin
  if index >= FPrevEditor.GetVerbCount then
  begin
    QBEVerbIndex:=GetQBEVerbIndex;
    if (QBEVerbIndex<>-1) and (index=FPrevEditor.GetVerbCount) then
      FQueryEditor.ExecuteVerb(QBEVerbIndex)
    else
    with TClientDataSetQuery(Component) do
    begin
         UpdateSQL.DataSet := Query;
         if EditUpdateSQL(UpdateSQL) then
            Self.Designer.Modified;
    end
  end else
      FPrevEditor.ExecuteVerb(Index);
end;

function TClientDataSetQueryEditor.GetVerb(Index: Integer): string;
begin
  if index >= FPrevEditor.GetVerbCount then
  begin
    if (GetQBEVerbIndex<>-1) and (index=FPrevEditor.GetVerbCount) then
      Result:=QBEVerb
    else Result :=  SUpdateSQLEditor;
  end else
    Result := FPrevEditor.GetVerb(Index);
end;

function TClientDataSetQueryEditor.GetVerbCount: Integer;
begin
  Result := FPrevEditor.GetVerbCount + 1;
  if GetQBEVerbIndex<>-1 then
     Inc(Result);
end;


procedure Register;
begin
  RegisterComponents('Midas', [TClientDataSetQuery]);
  RegisterComponentEditor(TClientDataSetQuery, TClientDataSetQueryEditor);
  RegisterPropertyEditor(TypeInfo(TParams), TClientDataSetQuery, '', TParamsProperty);
  RegisterPropertyEditor(TypeInfo(string), TClientDataSetQuery, 'DatabaseName',
    TDatabaseNameProperty);
  RegisterPropertyEditor(TypeInfo(string), TClientDataSetQuery, 'SessionName',
    TSessionNameProperty);
end;

procedure FindEditor;
var
  ds : TClientDataSet;
  dq : TQuery;
begin
  ds := TClientDataSet.Create(nil);
  try
    with GetComponentEditor(ds, nil) do
    begin
      TClass(PrevDataSetEditor):= ClassType;
      Free;
    end;
  finally
    ds.Free;
  end;
  dq:=TQuery.Create(nil);
  try
     with GetComponentEditor(dq, nil) do
     begin
          TClass(QueryEditor):=ClassType;
          Free;
     end;
  finally
    dq.Free;
  end;
end;

initialization
{$IFDEF VER120}
  QBEVerb := SGQEVerb;
{$ELSE}
  QBEVerb := SQBEVerb;
{$ENDIF}

  FindEditor;

end.
